package com.dsic.gj_erp.service.txzz.jhgl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dsic.gj_erp.bean.txzz.jhgl.TpbhPlan;
import com.dsic.gj_erp.mapper.txzz.jhgl.TpbhPlanMapper;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class TpbhPlanService extends ServiceImpl<TpbhPlanMapper, TpbhPlan> {


    /**
     * 提交时数据处理
     */
    @Transactional(rollbackFor = Exception.class)
    public void submitHandler(List<TpbhPlan> list,String userName,String userDeptCode){
        String data= DateUtil.date().toString("yyyy/MM/dd");
        List<TpbhPlan> subList=list.stream().filter(TpbhPlan::canSubmit)
                .filter(TpbhPlan::canSave)
                .map(l-> {
                    TpbhPlan plan=new TpbhPlan();
                    plan.setId(l.getId());
                    plan.updateForSubmit(userName,userDeptCode,data);
                    return plan;
                })
                .collect(Collectors.toList());
        this.updateBatchById(subList);
    }

    /**
     * excel导入时处理数据
     */
    public Map<String, List<TpbhPlan>> excelHandler(List<TpbhPlan> list){
        list=list.stream()
                .filter(TpbhPlan::checkBaseParam)
                .peek(l-> {
                    l.setStatus("01");
                    l.setDepartCode("01");
                }).collect(Collectors.toList());
        //查询重复数据
        List<TpbhPlan> repeatList= list.stream()
                .map(l-> this.getOne(new QueryWrapper<TpbhPlan>()
                        .eq("depart_code",l.getDepartCode())
                        .eq("project",l.getProject())
                        .eq("dwg_no",l.getDwgNo())
                        .last("limit 1")
        )).collect(Collectors.toList());

        //去重,这里自定义判断方法,先不重写equals
        //todo equals
        list=list.stream().filter(l->{
                for (TpbhPlan plan:repeatList){
                    if (l.equals(plan)){
                        return false;
                    }
                }
            return true;
        }).collect(Collectors.toList());

        Map<String,List<TpbhPlan>> map=new HashMap<>();
        map.put("successData",list);
        map.put("errorData",repeatList);
        return map;
    }

    /**
     * 提取
     */
//    public List<DmMfgDetail> extract(String project, String dwgNo){
//        DzglQueryContent content=new DzglQueryContent();
//        content.setProject(project);
//        //需要资料审核后才可提取
//        content.setStatus("02");
//        List<DmMfgDetail> list=dmMfgDetailService.getDetailByStatus(content);
//
//        //配送计划存在时也要提取
//        List<ImPmlPlan> pmlList=pmlPlanService.list(new QueryWrapper<ImPmlPlan>()
//                .eq("project",project)
//                .eq("depart_code","01")
//                .eq("ps_status","04")
//        );
//        pmlList.forEach(p-> list.forEach(l->{
//            if (p.getProject().equals(l.getProject())&&p.getDwgNo().equals(l.getDwgNo())){
//                l.setPmlZz(p.getPmlNo());
//            }
//        }));
//        return list;
//    }

    /**
     * 审核&审批
     */
    public void examine(List<TpbhPlan> list,String userName,String userCode,String action){
        String data=DateUtil.date().toString("yyyy/MM/dd");
        List<TpbhPlan> updateList=list.stream().map(l->{
            TpbhPlan plan=new TpbhPlan();
            plan.setId(l.getId());
            switch (action){
                case "sh":
                    plan.updateForSh(userName,userCode,data);
                    break;
                case "sp":
                    plan.updateForSp(userName,userCode,data);
            }
            return plan;
        }).collect(Collectors.toList());
        this.updateBatchById(updateList);
    }
}
